/***************************************************************************//**
* \file Accel_Mag_I2C.c
* \version 3.20
*
* \brief
*  This file provides the source code to the API for the SCB Component in
*  I2C mode.
*
* Note:
*
*******************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Accel_Mag_PVT.h"
#include "Accel_Mag_I2C_PVT.h"


/***************************************
*      I2C Private Vars
***************************************/

volatile uint8 Accel_Mag_state;  /* Current state of I2C FSM */

#if(Accel_Mag_SCB_MODE_UNCONFIG_CONST_CFG)

    /***************************************
    *  Configuration Structure Initialization
    ***************************************/

    /* Constant configuration of I2C */
    const Accel_Mag_I2C_INIT_STRUCT Accel_Mag_configI2C =
    {
        Accel_Mag_I2C_MODE,
        Accel_Mag_I2C_OVS_FACTOR_LOW,
        Accel_Mag_I2C_OVS_FACTOR_HIGH,
        Accel_Mag_I2C_MEDIAN_FILTER_ENABLE,
        Accel_Mag_I2C_SLAVE_ADDRESS,
        Accel_Mag_I2C_SLAVE_ADDRESS_MASK,
        Accel_Mag_I2C_ACCEPT_ADDRESS,
        Accel_Mag_I2C_WAKE_ENABLE,
        Accel_Mag_I2C_BYTE_MODE_ENABLE,
        Accel_Mag_I2C_DATA_RATE,
        Accel_Mag_I2C_ACCEPT_GENERAL_CALL,
    };

    /*******************************************************************************
    * Function Name: Accel_Mag_I2CInit
    ****************************************************************************//**
    *
    *
    *  Configures the Accel_Mag for I2C operation.
    *
    *  This function is intended specifically to be used when the Accel_Mag 
    *  configuration is set to Unconfigured Accel_Mag in the customizer. 
    *  After initializing the Accel_Mag in I2C mode using this function, 
    *  the component can be enabled using the Accel_Mag_Start() or 
    * Accel_Mag_Enable() function.
    *  This function uses a pointer to a structure that provides the configuration 
    *  settings. This structure contains the same information that would otherwise 
    *  be provided by the customizer settings.
    *
    *  \param config: pointer to a structure that contains the following list of 
    *   fields. These fields match the selections available in the customizer. 
    *   Refer to the customizer for further description of the settings.
    *
    *******************************************************************************/
    void Accel_Mag_I2CInit(const Accel_Mag_I2C_INIT_STRUCT *config)
    {
        uint32 medianFilter;
        uint32 locEnableWake;

        if(NULL == config)
        {
            CYASSERT(0u != 0u); /* Halt execution due to bad function parameter */
        }
        else
        {
            /* Configure pins */
            Accel_Mag_SetPins(Accel_Mag_SCB_MODE_I2C, Accel_Mag_DUMMY_PARAM,
                                     Accel_Mag_DUMMY_PARAM);

            /* Store internal configuration */
            Accel_Mag_scbMode       = (uint8) Accel_Mag_SCB_MODE_I2C;
            Accel_Mag_scbEnableWake = (uint8) config->enableWake;
            Accel_Mag_scbEnableIntr = (uint8) Accel_Mag_SCB_IRQ_INTERNAL;

            Accel_Mag_mode          = (uint8) config->mode;
            Accel_Mag_acceptAddr    = (uint8) config->acceptAddr;

        #if (Accel_Mag_CY_SCBIP_V0)
            /* Adjust SDA filter settings. Ticket ID#150521 */
            Accel_Mag_SET_I2C_CFG_SDA_FILT_TRIM(Accel_Mag_EC_AM_I2C_CFG_SDA_FILT_TRIM);
        #endif /* (Accel_Mag_CY_SCBIP_V0) */

            /* Adjust AF and DF filter settings. Ticket ID#176179 */
            if (((Accel_Mag_I2C_MODE_SLAVE != config->mode) &&
                 (config->dataRate <= Accel_Mag_I2C_DATA_RATE_FS_MODE_MAX)) ||
                 (Accel_Mag_I2C_MODE_SLAVE == config->mode))
            {
                /* AF = 1, DF = 0 */
                Accel_Mag_I2C_CFG_ANALOG_FITER_ENABLE;
                medianFilter = Accel_Mag_DIGITAL_FILTER_DISABLE;
            }
            else
            {
                /* AF = 0, DF = 1 */
                Accel_Mag_I2C_CFG_ANALOG_FITER_DISABLE;
                medianFilter = Accel_Mag_DIGITAL_FILTER_ENABLE;
            }

        #if (!Accel_Mag_CY_SCBIP_V0)
            locEnableWake = (Accel_Mag_I2C_MULTI_MASTER_SLAVE) ? (0u) : (config->enableWake);
        #else
            locEnableWake = config->enableWake;
        #endif /* (!Accel_Mag_CY_SCBIP_V0) */

            /* Configure I2C interface */
            Accel_Mag_CTRL_REG     = Accel_Mag_GET_CTRL_BYTE_MODE  (config->enableByteMode) |
                                            Accel_Mag_GET_CTRL_ADDR_ACCEPT(config->acceptAddr)     |
                                            Accel_Mag_GET_CTRL_EC_AM_MODE (locEnableWake);

            Accel_Mag_I2C_CTRL_REG = Accel_Mag_GET_I2C_CTRL_HIGH_PHASE_OVS(config->oversampleHigh) |
                    Accel_Mag_GET_I2C_CTRL_LOW_PHASE_OVS (config->oversampleLow)                          |
                    Accel_Mag_GET_I2C_CTRL_S_GENERAL_IGNORE((uint32)(0u == config->acceptGeneralAddr))    |
                    Accel_Mag_GET_I2C_CTRL_SL_MSTR_MODE  (config->mode);

            /* Configure RX direction */
            Accel_Mag_RX_CTRL_REG      = Accel_Mag_GET_RX_CTRL_MEDIAN(medianFilter) |
                                                Accel_Mag_I2C_RX_CTRL;
            Accel_Mag_RX_FIFO_CTRL_REG = Accel_Mag_CLEAR_REG;

            /* Set default address and mask */
            Accel_Mag_RX_MATCH_REG    = ((Accel_Mag_I2C_SLAVE) ?
                                                (Accel_Mag_GET_I2C_8BIT_ADDRESS(config->slaveAddr) |
                                                 Accel_Mag_GET_RX_MATCH_MASK(config->slaveAddrMask)) :
                                                (Accel_Mag_CLEAR_REG));


            /* Configure TX direction */
            Accel_Mag_TX_CTRL_REG      = Accel_Mag_I2C_TX_CTRL;
            Accel_Mag_TX_FIFO_CTRL_REG = Accel_Mag_CLEAR_REG;

            /* Configure interrupt with I2C handler but do not enable it */
            CyIntDisable    (Accel_Mag_ISR_NUMBER);
            CyIntSetPriority(Accel_Mag_ISR_NUMBER, Accel_Mag_ISR_PRIORITY);
            (void) CyIntSetVector(Accel_Mag_ISR_NUMBER, &Accel_Mag_I2C_ISR);

            /* Configure interrupt sources */
        #if(!Accel_Mag_CY_SCBIP_V1)
            Accel_Mag_INTR_SPI_EC_MASK_REG = Accel_Mag_NO_INTR_SOURCES;
        #endif /* (!Accel_Mag_CY_SCBIP_V1) */

            Accel_Mag_INTR_I2C_EC_MASK_REG = Accel_Mag_NO_INTR_SOURCES;
            Accel_Mag_INTR_RX_MASK_REG     = Accel_Mag_NO_INTR_SOURCES;
            Accel_Mag_INTR_TX_MASK_REG     = Accel_Mag_NO_INTR_SOURCES;

            Accel_Mag_INTR_SLAVE_MASK_REG  = ((Accel_Mag_I2C_SLAVE) ?
                            (Accel_Mag_GET_INTR_SLAVE_I2C_GENERAL(config->acceptGeneralAddr) |
                             Accel_Mag_I2C_INTR_SLAVE_MASK) : (Accel_Mag_CLEAR_REG));

            Accel_Mag_INTR_MASTER_MASK_REG = ((Accel_Mag_I2C_MASTER) ?
                                                     (Accel_Mag_I2C_INTR_MASTER_MASK) :
                                                     (Accel_Mag_CLEAR_REG));

            /* Configure global variables */
            Accel_Mag_state = Accel_Mag_I2C_FSM_IDLE;

            /* Internal slave variables */
            Accel_Mag_slStatus        = 0u;
            Accel_Mag_slRdBufIndex    = 0u;
            Accel_Mag_slWrBufIndex    = 0u;
            Accel_Mag_slOverFlowCount = 0u;

            /* Internal master variables */
            Accel_Mag_mstrStatus     = 0u;
            Accel_Mag_mstrRdBufIndex = 0u;
            Accel_Mag_mstrWrBufIndex = 0u;
        }
    }

#else

    /*******************************************************************************
    * Function Name: Accel_Mag_I2CInit
    ****************************************************************************//**
    *
    *  Configures the SCB for the I2C operation.
    *
    *******************************************************************************/
    void Accel_Mag_I2CInit(void)
    {
    #if(Accel_Mag_CY_SCBIP_V0)
        /* Adjust SDA filter settings. Ticket ID#150521 */
        Accel_Mag_SET_I2C_CFG_SDA_FILT_TRIM(Accel_Mag_EC_AM_I2C_CFG_SDA_FILT_TRIM);
    #endif /* (Accel_Mag_CY_SCBIP_V0) */

        /* Adjust AF and DF filter settings. Ticket ID#176179 */
        Accel_Mag_I2C_CFG_ANALOG_FITER_ENABLE_ADJ;

        /* Configure I2C interface */
        Accel_Mag_CTRL_REG     = Accel_Mag_I2C_DEFAULT_CTRL;
        Accel_Mag_I2C_CTRL_REG = Accel_Mag_I2C_DEFAULT_I2C_CTRL;

        /* Configure RX direction */
        Accel_Mag_RX_CTRL_REG      = Accel_Mag_I2C_DEFAULT_RX_CTRL;
        Accel_Mag_RX_FIFO_CTRL_REG = Accel_Mag_I2C_DEFAULT_RX_FIFO_CTRL;

        /* Set default address and mask */
        Accel_Mag_RX_MATCH_REG     = Accel_Mag_I2C_DEFAULT_RX_MATCH;

        /* Configure TX direction */
        Accel_Mag_TX_CTRL_REG      = Accel_Mag_I2C_DEFAULT_TX_CTRL;
        Accel_Mag_TX_FIFO_CTRL_REG = Accel_Mag_I2C_DEFAULT_TX_FIFO_CTRL;

        /* Configure interrupt with I2C handler but do not enable it */
        CyIntDisable    (Accel_Mag_ISR_NUMBER);
        CyIntSetPriority(Accel_Mag_ISR_NUMBER, Accel_Mag_ISR_PRIORITY);
    #if(!Accel_Mag_I2C_EXTERN_INTR_HANDLER)
        (void) CyIntSetVector(Accel_Mag_ISR_NUMBER, &Accel_Mag_I2C_ISR);
    #endif /* (Accel_Mag_I2C_EXTERN_INTR_HANDLER) */

        /* Configure interrupt sources */
    #if(!Accel_Mag_CY_SCBIP_V1)
        Accel_Mag_INTR_SPI_EC_MASK_REG = Accel_Mag_I2C_DEFAULT_INTR_SPI_EC_MASK;
    #endif /* (!Accel_Mag_CY_SCBIP_V1) */

        Accel_Mag_INTR_I2C_EC_MASK_REG = Accel_Mag_I2C_DEFAULT_INTR_I2C_EC_MASK;
        Accel_Mag_INTR_SLAVE_MASK_REG  = Accel_Mag_I2C_DEFAULT_INTR_SLAVE_MASK;
        Accel_Mag_INTR_MASTER_MASK_REG = Accel_Mag_I2C_DEFAULT_INTR_MASTER_MASK;
        Accel_Mag_INTR_RX_MASK_REG     = Accel_Mag_I2C_DEFAULT_INTR_RX_MASK;
        Accel_Mag_INTR_TX_MASK_REG     = Accel_Mag_I2C_DEFAULT_INTR_TX_MASK;

        /* Configure global variables */
        Accel_Mag_state = Accel_Mag_I2C_FSM_IDLE;

    #if(Accel_Mag_I2C_SLAVE)
        /* Internal slave variable */
        Accel_Mag_slStatus        = 0u;
        Accel_Mag_slRdBufIndex    = 0u;
        Accel_Mag_slWrBufIndex    = 0u;
        Accel_Mag_slOverFlowCount = 0u;
    #endif /* (Accel_Mag_I2C_SLAVE) */

    #if(Accel_Mag_I2C_MASTER)
    /* Internal master variable */
        Accel_Mag_mstrStatus     = 0u;
        Accel_Mag_mstrRdBufIndex = 0u;
        Accel_Mag_mstrWrBufIndex = 0u;
    #endif /* (Accel_Mag_I2C_MASTER) */
    }
#endif /* (Accel_Mag_SCB_MODE_UNCONFIG_CONST_CFG) */


/*******************************************************************************
* Function Name: Accel_Mag_I2CStop
****************************************************************************//**
*
*  Resets the I2C FSM into the default state.
*
*******************************************************************************/
void Accel_Mag_I2CStop(void)
{
    Accel_Mag_state = Accel_Mag_I2C_FSM_IDLE;
}


#if(Accel_Mag_I2C_WAKE_ENABLE_CONST)
    /*******************************************************************************
    * Function Name: Accel_Mag_I2CSaveConfig
    ****************************************************************************//**
    *
    *  Enables Accel_Mag_INTR_I2C_EC_WAKE_UP interrupt source. This interrupt
    *  triggers on address match and wakes up device.
    *
    *******************************************************************************/
    void Accel_Mag_I2CSaveConfig(void)
    {
    #if (!Accel_Mag_CY_SCBIP_V0)
        #if (Accel_Mag_I2C_MULTI_MASTER_SLAVE_CONST && Accel_Mag_I2C_WAKE_ENABLE_CONST)
            /* Enable externally clocked address match if it was not enabled before.
            * This applicable only for Multi-Master-Slave. Ticket ID#192742 */
            if (0u == (Accel_Mag_CTRL_REG & Accel_Mag_CTRL_EC_AM_MODE))
            {
                /* Enable external address match logic */
                Accel_Mag_Stop();
                Accel_Mag_CTRL_REG |= Accel_Mag_CTRL_EC_AM_MODE;
                Accel_Mag_Enable();
            }
        #endif /* (Accel_Mag_I2C_MULTI_MASTER_SLAVE_CONST) */

        #if (Accel_Mag_SCB_CLK_INTERNAL)
            /* Disable clock to internal address match logic. Ticket ID#187931 */
            Accel_Mag_SCBCLK_Stop();
        #endif /* (Accel_Mag_SCB_CLK_INTERNAL) */
    #endif /* (!Accel_Mag_CY_SCBIP_V0) */

        Accel_Mag_SetI2CExtClkInterruptMode(Accel_Mag_INTR_I2C_EC_WAKE_UP);
    }


    /*******************************************************************************
    * Function Name: Accel_Mag_I2CRestoreConfig
    ****************************************************************************//**
    *
    *  Disables Accel_Mag_INTR_I2C_EC_WAKE_UP interrupt source. This interrupt
    *  triggers on address match and wakes up device.
    *
    *******************************************************************************/
    void Accel_Mag_I2CRestoreConfig(void)
    {
        /* Disable wakeup interrupt on address match */
        Accel_Mag_SetI2CExtClkInterruptMode(Accel_Mag_NO_INTR_SOURCES);

    #if (!Accel_Mag_CY_SCBIP_V0)
        #if (Accel_Mag_SCB_CLK_INTERNAL)
            /* Enable clock to internal address match logic. Ticket ID#187931 */
            Accel_Mag_SCBCLK_Start();
        #endif /* (Accel_Mag_SCB_CLK_INTERNAL) */
    #endif /* (!Accel_Mag_CY_SCBIP_V0) */
    }
#endif /* (Accel_Mag_I2C_WAKE_ENABLE_CONST) */


/* [] END OF FILE */
