/***************************************************************************//**
* \file Accel_Mag_PM.c
* \version 3.20
*
* \brief
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* \copyright
* Copyright 2013-2016, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "Accel_Mag.h"
#include "Accel_Mag_PVT.h"

#if(Accel_Mag_SCB_MODE_I2C_INC)
    #include "Accel_Mag_I2C_PVT.h"
#endif /* (Accel_Mag_SCB_MODE_I2C_INC) */

#if(Accel_Mag_SCB_MODE_EZI2C_INC)
    #include "Accel_Mag_EZI2C_PVT.h"
#endif /* (Accel_Mag_SCB_MODE_EZI2C_INC) */

#if(Accel_Mag_SCB_MODE_SPI_INC || Accel_Mag_SCB_MODE_UART_INC)
    #include "Accel_Mag_SPI_UART_PVT.h"
#endif /* (Accel_Mag_SCB_MODE_SPI_INC || Accel_Mag_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(Accel_Mag_SCB_MODE_UNCONFIG_CONST_CFG || \
   (Accel_Mag_SCB_MODE_I2C_CONST_CFG   && (!Accel_Mag_I2C_WAKE_ENABLE_CONST))   || \
   (Accel_Mag_SCB_MODE_EZI2C_CONST_CFG && (!Accel_Mag_EZI2C_WAKE_ENABLE_CONST)) || \
   (Accel_Mag_SCB_MODE_SPI_CONST_CFG   && (!Accel_Mag_SPI_WAKE_ENABLE_CONST))   || \
   (Accel_Mag_SCB_MODE_UART_CONST_CFG  && (!Accel_Mag_UART_WAKE_ENABLE_CONST)))

    Accel_Mag_BACKUP_STRUCT Accel_Mag_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: Accel_Mag_Sleep
****************************************************************************//**
*
*  Prepares the Accel_Mag component to enter Deep Sleep.
*  The Enable wakeup from Deep Sleep Mode selection has an influence on this 
*  function implementation:
*  - Checked: configures the component to be wakeup source from Deep Sleep.
*  - Unchecked: stores the current component state (enabled or disabled) and 
*    disables the component. See SCB_Stop() function for details about component 
*    disabling.
*
*  Call the Accel_Mag_Sleep() function before calling the 
*  CyPmSysDeepSleep() function. 
*  Refer to the PSoC Creator System Reference Guide for more information about 
*  power management functions and Low power section of this document for the 
*  selected mode.
*
*  This function should not be called before entering Sleep.
*
*******************************************************************************/
void Accel_Mag_Sleep(void)
{
#if(Accel_Mag_SCB_MODE_UNCONFIG_CONST_CFG)

    if(Accel_Mag_SCB_WAKE_ENABLE_CHECK)
    {
        if(Accel_Mag_SCB_MODE_I2C_RUNTM_CFG)
        {
            Accel_Mag_I2CSaveConfig();
        }
        else if(Accel_Mag_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            Accel_Mag_EzI2CSaveConfig();
        }
    #if(!Accel_Mag_CY_SCBIP_V1)
        else if(Accel_Mag_SCB_MODE_SPI_RUNTM_CFG)
        {
            Accel_Mag_SpiSaveConfig();
        }
        else if(Accel_Mag_SCB_MODE_UART_RUNTM_CFG)
        {
            Accel_Mag_UartSaveConfig();
        }
    #endif /* (!Accel_Mag_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        Accel_Mag_backup.enableState = (uint8) Accel_Mag_GET_CTRL_ENABLED;

        if(0u != Accel_Mag_backup.enableState)
        {
            Accel_Mag_Stop();
        }
    }

#else

    #if (Accel_Mag_SCB_MODE_I2C_CONST_CFG && Accel_Mag_I2C_WAKE_ENABLE_CONST)
        Accel_Mag_I2CSaveConfig();

    #elif (Accel_Mag_SCB_MODE_EZI2C_CONST_CFG && Accel_Mag_EZI2C_WAKE_ENABLE_CONST)
        Accel_Mag_EzI2CSaveConfig();

    #elif (Accel_Mag_SCB_MODE_SPI_CONST_CFG && Accel_Mag_SPI_WAKE_ENABLE_CONST)
        Accel_Mag_SpiSaveConfig();

    #elif (Accel_Mag_SCB_MODE_UART_CONST_CFG && Accel_Mag_UART_WAKE_ENABLE_CONST)
        Accel_Mag_UartSaveConfig();

    #else

        Accel_Mag_backup.enableState = (uint8) Accel_Mag_GET_CTRL_ENABLED;

        if(0u != Accel_Mag_backup.enableState)
        {
            Accel_Mag_Stop();
        }

    #endif /* defined (Accel_Mag_SCB_MODE_I2C_CONST_CFG) && (Accel_Mag_I2C_WAKE_ENABLE_CONST) */

#endif /* (Accel_Mag_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: Accel_Mag_Wakeup
****************************************************************************//**
*
*  Prepares the Accel_Mag component for Active mode operation after 
*  Deep Sleep.
*  The Enable wakeup from Deep Sleep Mode selection has influence on this 
*  function implementation:
*  - Checked: restores the component Active mode configuration.
*  - Unchecked: enables the component if it was enabled before enter Deep Sleep.
*
*  This function should not be called after exiting Sleep.
*
*  \sideeffect
*   Calling the Accel_Mag_Wakeup() function without first calling the 
*   Accel_Mag_Sleep() function may produce unexpected behavior.
*
*******************************************************************************/
void Accel_Mag_Wakeup(void)
{
#if(Accel_Mag_SCB_MODE_UNCONFIG_CONST_CFG)

    if(Accel_Mag_SCB_WAKE_ENABLE_CHECK)
    {
        if(Accel_Mag_SCB_MODE_I2C_RUNTM_CFG)
        {
            Accel_Mag_I2CRestoreConfig();
        }
        else if(Accel_Mag_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            Accel_Mag_EzI2CRestoreConfig();
        }
    #if(!Accel_Mag_CY_SCBIP_V1)
        else if(Accel_Mag_SCB_MODE_SPI_RUNTM_CFG)
        {
            Accel_Mag_SpiRestoreConfig();
        }
        else if(Accel_Mag_SCB_MODE_UART_RUNTM_CFG)
        {
            Accel_Mag_UartRestoreConfig();
        }
    #endif /* (!Accel_Mag_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        if(0u != Accel_Mag_backup.enableState)
        {
            Accel_Mag_Enable();
        }
    }

#else

    #if (Accel_Mag_SCB_MODE_I2C_CONST_CFG  && Accel_Mag_I2C_WAKE_ENABLE_CONST)
        Accel_Mag_I2CRestoreConfig();

    #elif (Accel_Mag_SCB_MODE_EZI2C_CONST_CFG && Accel_Mag_EZI2C_WAKE_ENABLE_CONST)
        Accel_Mag_EzI2CRestoreConfig();

    #elif (Accel_Mag_SCB_MODE_SPI_CONST_CFG && Accel_Mag_SPI_WAKE_ENABLE_CONST)
        Accel_Mag_SpiRestoreConfig();

    #elif (Accel_Mag_SCB_MODE_UART_CONST_CFG && Accel_Mag_UART_WAKE_ENABLE_CONST)
        Accel_Mag_UartRestoreConfig();

    #else

        if(0u != Accel_Mag_backup.enableState)
        {
            Accel_Mag_Enable();
        }

    #endif /* (Accel_Mag_I2C_WAKE_ENABLE_CONST) */

#endif /* (Accel_Mag_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
